/*
    This file implements a dialog box for selecting between installed aDIO
    devices.

    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------

    $Id: aDIO_Select.cpp 80941 2014-08-01 18:41:26Z mmcintire $
*/

#include "stdafx.h"
#include "aDIO_Select.h"
#include <windows.h>
#include <tchar.h>
#include <string>
#include <iostream>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CaDIO_Select dialog
/////////////////////////////////////////////////////////////////////////////

CaDIO_Select::CaDIO_Select(CWnd* pParent /*=NULL*/)
    : CDialog(CaDIO_Select::IDD, pParent)
{
    //{{AFX_DATA_INIT(CaDIO_Select)
        // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_EnumMax = ADIO_INTRFC_MAX_DEVICE_COUNT;
    m_ValidInfo = false;
}


void CaDIO_Select::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CaDIO_Select)
    DDX_Control(pDX, IDC_AS_ADIO_LIST, m_aDIO_List);
    //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CaDIO_Select, CDialog)
    //{{AFX_MSG_MAP(CaDIO_Select)
    ON_BN_CLICKED(ID_AS_OK, OnOK)
    ON_BN_CLICKED(ID_AS_CANCEL, OnCancel)
    ON_BN_CLICKED(IDC_AS_BUTTON_REFRESH, OnBTNRefresh)
    ON_LBN_SELCHANGE(IDC_AS_ADIO_LIST, OnSelchangeLST)
    //}}AFX_MSG_MAP
ON_LBN_DBLCLK(IDC_AS_ADIO_LIST, OnLbnDblclkLst)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CaDIO_Select message handlers
/////////////////////////////////////////////////////////////////////////////

BOOL CaDIO_Select::OnInitDialog()
{
    CDialog::OnInitDialog();

    HICON m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON_RTD);
    SetIcon(m_hIcon, TRUE);         // Set big icon
    SetIcon(m_hIcon, FALSE);        // Set small icon

    OnBTNRefresh();

    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
}

void CaDIO_Select::OnBTNRefresh()
{
    aDIO_Error result;
    char adio_desc[200];
    char avail_buff[20], irq_buff[20], wake_buff[20];

    GetDlgItem(IDC_AS_BUTTON_REFRESH)->EnableWindow(FALSE);
    GetDlgItem(ID_AS_OK)->EnableWindow(FALSE);

    // clear list
    m_aDIO_List.ResetContent();


    // Enumerate as many devices as possible
    result = aDIO_General_Enum(m_EnumList, m_EnumMax);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        GetDlgItem(IDC_AS_BUTTON_REFRESH)->EnableWindow();
        return;
    }

    for(int i = 0; i < m_EnumMax; ++i)
    {
        if(m_EnumList[i].availability == ADIO_INTRFC_AVAIL_INVALID)
        {
            break;
        }
        else
        {
            switch(m_EnumList[i].availability)
            {
                case ADIO_INTRFC_AVAIL_MALFUNC:
                {
                    sprintf_s(avail_buff, 20, "Malfunc.");
                    break;
                }
                case ADIO_INTRFC_AVAIL_IN_USE:
                {
                    sprintf_s(avail_buff, 20, "In Use  ");
                    break;
                }
                case ADIO_INTRFC_AVAIL_READY:
                {
                    sprintf_s(avail_buff, 20, "Ready   ");
                    break;
                }
                default:
                {
                    sprintf_s(avail_buff, 20, "Invalid ");
                    break;
                }
            }

            if(m_EnumList[i].irq == 0)
            {
                sprintf_s(irq_buff, 20, "     None");
                sprintf_s(wake_buff, 20, "No");
            }
            else
            {
                sprintf_s(irq_buff, 20, "%8d ", m_EnumList[i].irq);

                if(m_EnumList[i].has_wake)
                    sprintf_s(wake_buff, 20, "Yes");
                else
                    sprintf_s(wake_buff, 20, "No");
            }

            sprintf_s(  adio_desc,
                        200,
                        "%3d        %s   0x%03X%s%11s",
                        m_EnumList[i].dev_num,
                        avail_buff,
                        m_EnumList[i].base_address,
                        irq_buff,
                        wake_buff);


            m_aDIO_List.AddString(adio_desc);

        }
    }

    GetDlgItem(IDC_AS_BUTTON_REFRESH)->EnableWindow();
}

// Handle Select button
void CaDIO_Select::OnOK()
{
    int select = m_aDIO_List.GetCurSel();
    int found = -1;

    m_ValidInfo = false;


    if(select == LB_ERR)
    {
        return;
    }

    for(int i = 0; i < m_EnumMax; ++i)
    {
        if(m_EnumList[i].availability != ADIO_INTRFC_AVAIL_INVALID)
            found++;
        if(found == select)
        {
            m_DeviceInfo = m_EnumList[i];
            m_ValidInfo = true;
            break;
        }
    }

    CDialog::OnOK();
}

// Handle Exit Button
void CaDIO_Select::OnCancel()
{
    m_ValidInfo = false;

    CDialog::OnCancel();
}

void CaDIO_Select::OnSelchangeLST()
{
    if( m_aDIO_List.GetCurSel() != LB_ERR )
        GetDlgItem(ID_AS_OK)->EnableWindow();
}

void CaDIO_Select::OnLbnDblclkLst()
{
    OnSelchangeLST();
    if(GetDlgItem(ID_AS_OK)->IsWindowEnabled())
        OnOK();
}


