/*
    aDIO Interrupt Mode Example

    Please see ReadMe.txt for a description of this example, including
    instructions on setup and use.
*/

#include "stdafx.h"
#include "Interrupts.h"
#include "InterruptsDlg.h"
#include "aDIO_Select.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CInterruptsDlg dialog


// Timer id
#define TIMER_0     0x1000

// Time (in milliseconds) between timer events
#define DELAY_TIME  200


// GLOBALS ///////////////////////////////////////////////////////////////////

// A critical section for synchronization between callback and dialog
CRITICAL_SECTION crit_section;

uint32 Int_Count = 0;
aDIO_Error callback_status = ADIO_ERROR_CODE_SUCCESS;

//////////////////////////////////////////////////////////////////////////////

// The interrupt callback function
void aDIO_callback(aDIO_Interrupt_Callback_Info Info)
{
    char buff[400];
    char* int_name;
    bool do_msg = false;

    EnterCriticalSection(&crit_section);

    callback_status = Info.result;

    // Only record the port info if this was a successful callback
    if(callback_status == ADIO_ERROR_CODE_SUCCESS)
    {
        Int_Count += Info.interrupt_count;
        do_msg = true;
    }

    LeaveCriticalSection(&crit_section);

    // If the callback was successful, show a relevant message
    if(do_msg)
    {
        if(Info.type == ADIO_INTERRUPT_INT_TYPE_STROBE)
            int_name = "A Strobe";
        else if(Info.type == ADIO_INTERRUPT_INT_TYPE_EVENT)
            int_name = "An Event";
        else if(Info.type == ADIO_INTERRUPT_INT_TYPE_MATCH)
            int_name = "A Match";

        sprintf_s(  buff,
                    400,
                    "%s interrupt occurred!\n\n"
                        "Values at the time of the interrupt were:\n"
                        "  Compare Reg: 0x%02X\n"
                        "  Port 0: 0x%02X\n"
                        "  Port 1: 0x%02X",
                    int_name,
                    Info.compare_val,
                    Info.port0_val,
                    Info.port1_val);

        MessageBox(NULL, buff, "Interrupt Received", MB_OK);
    }
}



CInterruptsDlg::CInterruptsDlg(CWnd* pParent /*=NULL*/)
    : CDialog(CInterruptsDlg::IDD, pParent)
{
    m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON_RTD);

    m_Handle = NULL;

    m_P1_Output = 0x00;
    m_Mask_Val = 0x00;
    m_Compare_Val = 0x00;

    m_Int_Enabled = false;
}

void CInterruptsDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);

    DDX_Control(pDX, IDC_EDIT_MASK, m_Mask_Edit);
    DDX_Control(pDX, IDC_BUTTON_SET_MASK, m_Mask_Box);

    DDX_Control(pDX, IDC_EDIT_COMPARE, m_Compare_Edit);
    DDX_Control(pDX, IDC_BUTTON_SET_COMPARE, m_Compare_Box);

    DDX_Control(pDX, IDC_EDIT_PORT1, m_Port1_Edit);
    DDX_Control(pDX, IDC_BUTTON_SET_PORT1, m_Port1_Box);

    DDX_Control(pDX, IDC_EDIT_PORT0, m_Port0_Edit);

    DDX_Control(pDX, IDC_STATIC_INT_COUNT, m_Count_Text);

    DDX_Control(pDX, IDC_RADIO_DISABLED, m_Disabled_Radio);
    DDX_Control(pDX, IDC_RADIO_STROBE, m_Strobe_Radio);
    DDX_Control(pDX, IDC_RADIO_EVENT, m_Event_Radio);
    DDX_Control(pDX, IDC_RADIO_MATCH, m_Match_Radio);
}

BEGIN_MESSAGE_MAP(CInterruptsDlg, CDialog)
    ON_EN_CHANGE(IDC_EDIT_MASK, OnMaskChange)
    ON_BN_CLICKED(IDC_BUTTON_SET_MASK, OnSetMask)
    ON_EN_CHANGE(IDC_EDIT_COMPARE, OnCompareChange)
    ON_BN_CLICKED(IDC_BUTTON_SET_COMPARE, OnSetCompare)
    ON_EN_CHANGE(IDC_EDIT_PORT1, OnPort1Change)
    ON_BN_CLICKED(IDC_BUTTON_SET_PORT1, OnSetPort1)
    ON_BN_CLICKED(IDC_RADIO_DISABLED, OnRadioDisabled)
    ON_BN_CLICKED(IDC_RADIO_STROBE, OnRadioStrobe)
    ON_BN_CLICKED(IDC_RADIO_EVENT, OnRadioEvent)
    ON_BN_CLICKED(IDC_RADIO_MATCH, OnRadioMatch)
    ON_BN_CLICKED(IDCLOSE, OnClose)
    ON_WM_TIMER()
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()


// CInterruptsDlg message handlers

BOOL CInterruptsDlg::OnInitDialog()
{
    CaDIO_Select adio_select;
    aDIO_Error result;
    char* err_msg = NULL;
    uint8 direction;

    CDialog::OnInitDialog();

    // Set the icon for this dialog.  The framework does this automatically
    //  when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);         // Set big icon
    SetIcon(m_hIcon, FALSE);        // Set small icon


    // Set up the user interface
    m_Mask_Edit.SetLimitText(2);
    m_Mask_Edit.SetWindowText("00");
    m_Mask_Edit.EnableWindow(FALSE);
    m_Mask_Box.EnableWindow(FALSE);

    m_Compare_Edit.SetLimitText(2);
    m_Compare_Edit.SetWindowText("00");
    m_Compare_Edit.EnableWindow(FALSE);
    m_Compare_Box.EnableWindow(FALSE);

    m_Port1_Edit.SetLimitText(2);
    m_Port1_Edit.SetWindowText("00");
    m_Port1_Edit.EnableWindow(FALSE);
    m_Port1_Box.EnableWindow(FALSE);

    m_Port0_Edit.SetWindowText("00");

    m_Count_Text.SetWindowText("0");

    m_Disabled_Radio.SetCheck(BST_CHECKED);
    m_Strobe_Radio.SetCheck(BST_UNCHECKED);
    m_Event_Radio.SetCheck(BST_UNCHECKED);
    m_Match_Radio.SetCheck(BST_UNCHECKED);


    // Let user select the aDIO to use
    do
    {
        adio_select.DoModal();

        if(!adio_select.m_ValidInfo)
        {
            EndDialog(IDCANCEL);
            return FALSE;
        }

        if(adio_select.m_DeviceInfo.availability != ADIO_INTRFC_AVAIL_READY)
        {
            MessageBox( "The aDIO device selected cannot be used.\nTry again.",
                        "Device Unavailable",
                        MB_ICONERROR | MB_OK);
        }
        else if(adio_select.m_DeviceInfo.irq == 0)
        {
            MessageBox(
                "The aDIO device selected does not have an IRQ.\nTry again.",
                "Device Unavailable",
                MB_ICONERROR | MB_OK);
        }
        else
        {
            break;
        }

    } while(TRUE);


    // Open and setup the aDIO
    do
    {
        // Open the aDIO
        result = aDIO_General_Open(adio_select.m_DeviceInfo, &m_Handle);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Opening the aDIO failed with the following message:\n";
            break;
        }

        // Prepare the critical section
        InitializeCriticalSection(&crit_section);

        // Reset the aDIO
        result = aDIO_General_Reset(m_Handle);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Resetting the aDIO failed with the following message:\n";
            break;
        }

        // Set the direction of the ports
        direction = ADIO_PORT_BITS_ALL;
        direction = ~direction;
        result = aDIO_Port_SetDirection(m_Handle,
                                        direction,
                                        ADIO_PORT_DIR_P1_OUTPUT);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Setting Port directions failed "
                        "with the following message:\n";
            break;
        }

        // Set the interrupt configuration
        if(!UpdateInterruptConfig(false))
            break;

        // Set the mask
        if(!UpdateMask(false))
            break;

        // Set port 1's output
        if(!UpdateP1Output(false))
            break;

        // Install the same callback for all interrupt types
        result = aDIO_Interrupt_InstallCallback(m_Handle,
                                                ADIO_INTERRUPT_INT_TYPE_STROBE,
                                                aDIO_callback,
                                                NULL);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Installing the strobe callback failed "
                        "with the following message:\n";
            break;
        }

        result = aDIO_Interrupt_InstallCallback(m_Handle,
                                                ADIO_INTERRUPT_INT_TYPE_EVENT,
                                                aDIO_callback,
                                                NULL);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Installing the event callback failed "
                        "with the following message:\n";
            break;
        }

        result = aDIO_Interrupt_InstallCallback(m_Handle,
                                                ADIO_INTERRUPT_INT_TYPE_MATCH,
                                                aDIO_callback,
                                                NULL);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Installing the match callback failed "
                        "with the following message:\n";
            break;
        }

        SetTimer(TIMER_0, DELAY_TIME, NULL);

        return TRUE;  // return TRUE unless you set the focus to a control

    } while(0);

    // If we're here there was a failure of some kind
    Cleanup();
    if(err_msg != NULL)
        Fail(result, err_msg, false);
    EndDialog(IDCANCEL);

    return FALSE;
}


bool CInterruptsDlg::UpdateMask(bool quit)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    uint8 mask;

    // Set the Mask
    result = aDIO_Port_SetMask(m_Handle, m_Mask_Val);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Setting the Mask value failed with the following message:\n",
                quit);
        return false;
    }

    // Read it back and make sure it was set correctly
    result = aDIO_Port_GetMask(m_Handle, &mask);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Getting the Mask value failed with the following message:\n",
                quit);
        return false;
    }

    if(mask != m_Mask_Val)
    {
        Fail(   ADIO_ERROR_CODE_SUCCESS,
                "The Mask value read didn't match the one set.\n",
                quit);
        return false;
    }

    return true;
}

bool CInterruptsDlg::UpdateCompare(bool quit)
{
    aDIO_Error result;
    uint8 compare;

    // Set the Compare value
    result = aDIO_Port_SetCompare(m_Handle, m_Compare_Val);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Setting the Compare value failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    // Read it back and make sure it was set correctly
    result = aDIO_Port_GetCompare(m_Handle, &compare);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Getting the Compare value failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    if(compare != m_Compare_Val)
    {
        Fail(   ADIO_ERROR_CODE_SUCCESS,
                "The Compare value read didn't match the one set.\n",
                quit);
        return false;
    }

    return true;
}

bool CInterruptsDlg::UpdateP1Output(bool quit)
{
    aDIO_Error result;

    // Set Port 1's output
    result = aDIO_Port_WritePort(m_Handle, ADIO_PORT_DIO_PORT_1, m_P1_Output);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Setting Port 1 output failed with the following message:\n",
                quit);
        return false;
    }

    return true;
}

bool CInterruptsDlg::UpdateInterruptConfig(bool quit)
{
    aDIO_Error result;
    aDIO_Interrupt_Int_Type int_type;
    BOOL enable;

    // Set the new interrupt configuration
    result = aDIO_Interrupt_SetConfig(  m_Handle,
                                        m_Int_Enabled,
                                        m_Int_Type,
                                        FALSE);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Configuring interrupts failed with the following message:\n",
                quit);
        return false;
    }

    // Read the interrupt configuration
    result = aDIO_Interrupt_GetConfig(m_Handle, &enable, &int_type, NULL);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Getting interrupt configuration failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    // Check the interrupt configuration
    if( (enable && !m_Int_Enabled) ||
        (!enable && m_Int_Enabled) ||
        (enable && m_Int_Enabled && (int_type != m_Int_Type)))
    {
        Fail(   ADIO_ERROR_CODE_SUCCESS,
                "The interrupt was not properly configured.\n",
                quit);
        return false;
    }

    return true;
}



void CInterruptsDlg::OnMaskChange()
{
    m_Mask_Box.EnableWindow();
}

void CInterruptsDlg::OnSetMask()
{
    uint8 value;
    char buff[5];

    m_Mask_Box.EnableWindow(FALSE);

    if(!GetHexByteFromEdit(&m_Mask_Edit, &value))
    {
        MessageBox( "The Mask value you entered is invalid.\nTry again.",
                    "Invalid Input",
                    MB_ICONERROR | MB_OK);

        // Return the mask edit box to the mask's actual value
        sprintf_s(buff, 5, "%02X", m_Mask_Val);
        m_Mask_Edit.SetWindowText(buff);
        m_Mask_Box.EnableWindow(FALSE);

        return;
    }

    // Update the internal variable
    m_Mask_Val = value;

    // Update the mask
    UpdateMask();
}

void CInterruptsDlg::OnCompareChange()
{
    m_Compare_Box.EnableWindow();
}

void CInterruptsDlg::OnSetCompare()
{
    uint8 value;
    char buff[5];

    m_Compare_Box.EnableWindow(FALSE);

    if(!GetHexByteFromEdit(&m_Compare_Edit, &value))
    {
        MessageBox( "The Compare value you entered is invalid.\nTry again.",
                    "Invalid Input",
                    MB_ICONERROR | MB_OK);

        // Return the compare edit box to the compare's actual value
        sprintf_s(buff, 5, "%02X", m_Compare_Val);
        m_Compare_Edit.SetWindowText(buff);
        m_Compare_Box.EnableWindow(FALSE);

        return;
    }

    // Update the internal variable
    m_Compare_Val = value;

    // Update the compare
    UpdateCompare();
}

void CInterruptsDlg::OnPort1Change()
{
    m_Port1_Box.EnableWindow();
}

void CInterruptsDlg::OnSetPort1()
{
    uint8 value;
    char buff[5];

    m_Port1_Box.EnableWindow(FALSE);

    if(!GetHexByteFromEdit(&m_Port1_Edit, &value))
    {
        MessageBox(
                "The Port 1 output value you entered is invalid.\nTry again.",
                "Invalid Input",
                MB_ICONERROR | MB_OK);

        // Return the port 1 edit box to port 1's actual value
        sprintf_s(buff, 5, "%02X", m_P1_Output);
        m_Port1_Edit.SetWindowText(buff);
        m_Port1_Box.EnableWindow(FALSE);

        return;
    }

    // Update the internal variable
    m_P1_Output = value;

    // Update port 1's output
    UpdateP1Output();
}

void CInterruptsDlg::OnRadioDisabled()
{
    // Uncheck other radio buttons
    m_Strobe_Radio.SetCheck(BST_UNCHECKED);
    m_Event_Radio.SetCheck(BST_UNCHECKED);
    m_Match_Radio.SetCheck(BST_UNCHECKED);

    // Disable other controls
    m_Mask_Edit.EnableWindow(FALSE);
    m_Mask_Box.EnableWindow(FALSE);

    m_Compare_Edit.EnableWindow(FALSE);
    m_Compare_Box.EnableWindow(FALSE);

    m_Port1_Edit.EnableWindow(FALSE);
    m_Port1_Box.EnableWindow(FALSE);

    // Update the internal variable
    m_Int_Enabled = false;

    // Update the interrupt config
    UpdateInterruptConfig();
}

void CInterruptsDlg::OnRadioStrobe()
{
    // Uncheck other radio buttons
    m_Disabled_Radio.SetCheck(BST_UNCHECKED);
    m_Event_Radio.SetCheck(BST_UNCHECKED);
    m_Match_Radio.SetCheck(BST_UNCHECKED);

    // Disable/Enable other controls
    m_Mask_Edit.EnableWindow(FALSE);
    m_Mask_Box.EnableWindow(FALSE);

    m_Compare_Edit.EnableWindow(FALSE);
    m_Compare_Box.EnableWindow(FALSE);

    m_Port1_Edit.EnableWindow();
    m_Port1_Box.EnableWindow();

    // Update the internal variable
    m_Int_Enabled = true;
    m_Int_Type = ADIO_INTERRUPT_INT_TYPE_STROBE;

    // Update the interrupt config
    UpdateInterruptConfig();
}

void CInterruptsDlg::OnRadioEvent()
{
    // Uncheck other radio buttons
    m_Disabled_Radio.SetCheck(BST_UNCHECKED);
    m_Strobe_Radio.SetCheck(BST_UNCHECKED);
    m_Match_Radio.SetCheck(BST_UNCHECKED);

    // Disable other controls
    m_Mask_Edit.EnableWindow();
    m_Mask_Box.EnableWindow();

    m_Compare_Edit.EnableWindow(FALSE);
    m_Compare_Box.EnableWindow(FALSE);

    m_Port1_Edit.EnableWindow();
    m_Port1_Box.EnableWindow();

    // Update the internal variable
    m_Int_Enabled = true;
    m_Int_Type = ADIO_INTERRUPT_INT_TYPE_EVENT;

    // Update the interrupt config
    UpdateInterruptConfig();
}

void CInterruptsDlg::OnRadioMatch()
{
    // Uncheck other radio buttons
    m_Disabled_Radio.SetCheck(BST_UNCHECKED);
    m_Strobe_Radio.SetCheck(BST_UNCHECKED);
    m_Event_Radio.SetCheck(BST_UNCHECKED);

    // Disable other controls
    m_Mask_Edit.EnableWindow();
    m_Mask_Box.EnableWindow();

    m_Compare_Edit.EnableWindow();
    m_Compare_Box.EnableWindow();

    m_Port1_Edit.EnableWindow();
    m_Port1_Box.EnableWindow();

    // Update the internal variable
    m_Int_Enabled = true;
    m_Int_Type = ADIO_INTERRUPT_INT_TYPE_MATCH;

    // Update the interrupt config
    UpdateInterruptConfig();

    // Now the Compare value can be set
    UpdateCompare();
}

void CInterruptsDlg::OnTimer(UINT_PTR nIDEvent)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    uint32 int_count;
    uint8 value;
    char* err_msg;
    char buff[20];

    if(nIDEvent != TIMER_0)
    {
        CDialog::OnTimer(nIDEvent);
        return;
    }

    do
    {
        EnterCriticalSection(&crit_section);
        result = callback_status;
        int_count = Int_Count;
        LeaveCriticalSection(&crit_section);

        // Check for an error in the callback
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "The callback thread failed "
                        "with the following message:\n";
            break;
        }

        // Update the interrupt count
        sprintf_s(buff, 20, "%d", int_count);
        m_Count_Text.SetWindowText(buff);

        // 1ms delay needed for back-to-back read/write operations on new cpuModules.
        // Consult the aDIO section of your hardware manual for more information.
        Delay(1);
        
        // Update the Port 0 value
        result = aDIO_Port_ReadPort(m_Handle, ADIO_PORT_DIO_PORT_0, &value);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Reading Port 0 failed with the following message:\n";
            break;
        }

        sprintf_s(buff, 20, "%02X", value);
        m_Port0_Edit.SetWindowText(buff);

    } while(0);

    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        KillTimer(TIMER_0);
        Fail(result, err_msg);
    }

    CDialog::OnTimer(nIDEvent);
}

// Handle a Close action
void CInterruptsDlg::OnClose()
{
    Cleanup();
    CDialog::OnCancel();
}

// Use the enter key to set the values in the text fields
void CInterruptsDlg::OnOK(void){
    CWnd* pwndCtrl = GetFocus();
    int ctrl_id = pwndCtrl->GetDlgCtrlID();
    
    switch(ctrl_id){
        case IDC_EDIT_MASK:
            OnSetMask();
            break;
        case IDC_EDIT_COMPARE:
            OnSetCompare();
            break;
        case IDC_EDIT_PORT1:
            OnSetPort1();
            break;
        case IDOK:
            CDialog::OnOK();
            break;
        default:
            break;
    }
}

// Do "clean-up" before dialog finishes
void CInterruptsDlg::Cleanup()
{
    aDIO_Error result;

    // If m_Handle is NULL, the board hasn't been opened or clean up has already
    //  been done
    if(!m_Handle)
        return;

    KillTimer(TIMER_0);

    // Remove the callbacks
    result = aDIO_Interrupt_RemoveCallback( m_Handle,
                                            ADIO_INTERRUPT_INT_TYPE_STROBE);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Removing the strobe callback failed "
                    "with the following message:\n",
                false);
    }

    result = aDIO_Interrupt_RemoveCallback( m_Handle,
                                            ADIO_INTERRUPT_INT_TYPE_EVENT);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Removing the event callback failed "
                    "with the following message:\n",
                false);
    }

    result = aDIO_Interrupt_RemoveCallback( m_Handle,
                                            ADIO_INTERRUPT_INT_TYPE_MATCH);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Removing the match callback failed "
                    "with the following message:\n",
                false);
    }

    // Disable interrupts
    result = aDIO_Interrupt_SetConfig(  m_Handle,
                                        FALSE,
                                        (aDIO_Interrupt_Int_Type)0,
                                        FALSE);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Disabling interrupts failed with the following message:\n",
                false);
    }

    DeleteCriticalSection(&crit_section);

    // Close the aDIO
    result = aDIO_General_Close(m_Handle);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   ADIO_ERROR_CODE_SUCCESS,
                "Closing the aDIO failed. See the Application Log "\
                    "for the associated error message.",
                false);
    }

    // Mark that clean up is complete
    m_Handle = NULL;
}

// Displays an error message in a small window.
//
// error- If this is not ADIO_ERROR_CODE_SUCCESS, the error code's message is
//      appended to msg.
// msg - This is the example's explanation of the error
// quit - If true, clean up is done before displaying the error message and then
//      the program is exited after displaying the error message.
void CInterruptsDlg::Fail(aDIO_Error error, char* msg, bool quit)
{
    char final_msg[2048];
    char lib_msg[ADIO_ERROR_MAX_MSG_LENGTH];

    // If there was an error code given, get the last error message from the
    //  aDIO library otherwise just use the provided message.
    if(error != ADIO_ERROR_CODE_SUCCESS)
    {
        // Set a default message, in case getting the error message doesn't work
        sprintf_s(  lib_msg,
                    ADIO_ERROR_MAX_MSG_LENGTH,
                    "\nError message could not be found for error code 0x%X. "
                        "See the Application Log for details.\n",
                    error);

        if(m_Handle)
        {
            // Get the error message
            aDIO_General_GetLastErrorMsg(   m_Handle,
                                            lib_msg,
                                            ADIO_ERROR_MAX_MSG_LENGTH);
        }

        // Combine the retrieved message and the application's error message
        sprintf_s(final_msg, 2048, "%s\n%s", msg, lib_msg);
    }
    else
    {
        // Copy the general error message to the final buffer
        sprintf_s(final_msg, 2048, "%s", msg);
    }

    if(quit)
        Cleanup();

    MessageBox(final_msg, "Error - Aborting", MB_ICONERROR | MB_OK);

    if(quit)
        CDialog::OnCancel();
}


bool CInterruptsDlg::GetHexByteFromEdit(CEdit* edit, uint8* value)
{
    int numchars;
    char buff[4];
    char temp;

    numchars = edit->LineLength();

    if(numchars < 1 || numchars > 2)
        return false;

    if(!edit->GetWindowText(buff, 3))
        return false;

    *value = 0;

    for(int i = 0; i < numchars; ++i)
    {
        // Shift to the left
        *value *= 16;

        // Add on the next character
        temp = buff[i];

        if(temp >= 'a' && temp <= 'f')
        {
            *value += (temp - 'a') + 10;
        }
        else if(temp >= 'A' && temp <= 'F')
        {
            *value += (temp - 'A') + 10;
        }
        else if(temp >= '0' && temp <= '9')
        {
            *value += temp - '0';
        }
        else
            return false;
    }

    return true;
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CInterruptsDlg::OnPaint()
{
    if (IsIconic())
    {
        CPaintDC dc(this); // device context for painting

        SendMessage(WM_ICONERASEBKGND,
                    reinterpret_cast<WPARAM>(dc.GetSafeHdc()),
                    0);

        // Center icon in client rectangle
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;

        // Draw the icon
        dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
        CDialog::OnPaint();
    }
}

// The system calls this function to obtain the cursor to display while the
//  user drags the minimized window.
HCURSOR CInterruptsDlg::OnQueryDragIcon()
{
    return static_cast<HCURSOR>(m_hIcon);
}

// Busy-wait routine using the Microsoft QueryPerformanceCounter
void CInterruptsDlg::Delay(uint32 SleepTimeMS){
    
    LARGE_INTEGER start_count;
    LARGE_INTEGER counts_per_sec;
    LARGE_INTEGER current_count;
    double ticks_per_ms;
    double ticks_passed;
    
    if(QueryPerformanceFrequency(&counts_per_sec) == false){
        // QueryPerformanceFrequency failed. Use Sleep()
        Sleep(1000);
        return;
    }
    ticks_per_ms = static_cast<double>(counts_per_sec.QuadPart) / 1000.0;
    
    QueryPerformanceCounter(&start_count);
    
    while(1){
        // query the counter until at least SleepTimeMS has passed 
        QueryPerformanceCounter(&current_count);
        ticks_passed = static_cast<double>(current_count.QuadPart - start_count.QuadPart);
        if(ticks_passed >= (ticks_per_ms * SleepTimeMS)){
            break;
        }
    }
}

