/*
    Digital IO Port Example

    Please see ReadMe.txt for a description of this example, including
    instructions on setup and use.
*/

#include "stdafx.h"
#include "Ports.h"
#include "PortsDlg.h"
#include "aDIO_Select.h"
#include <windows.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CPortsDlg dialog



CPortsDlg::CPortsDlg(CWnd* pParent /*=NULL*/)
    : CDialog(CPortsDlg::IDD, pParent)
{
    m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON_RTD);

    m_Handle = NULL;
    m_P0_Output = 0x00;
    m_P1_Output = 0x00;
    m_P0_Direction = 0x00;
    m_P1_Direction = ADIO_PORT_DIR_P1_OUTPUT;
}

void CPortsDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);

    // Port 0
    DDX_Control(pDX, IDC_EDIT_P0_DIR, m_P0_Dir_Edit);
    DDX_Control(pDX, IDC_BUTTON_P0_DIRSET, m_P0_DirSet_Box);
    DDX_Control(pDX, IDC_EDIT_P0_OUT, m_P0_Out_Edit);
    DDX_Control(pDX, IDC_BUTTON_P0_OUTSET, m_P0_OutSet_Box);
    DDX_Control(pDX, IDC_EDIT_P0_IN, m_P0_In_Edit);

    // Port 1
    DDX_Control(pDX, IDC_BUTTON_P1_DIR, m_P1_Dir_Box);
    DDX_Control(pDX, IDC_EDIT_P1_OUT, m_P1_Out_Edit);
    DDX_Control(pDX, IDC_BUTTON_P1_OUTSET, m_P1_OutSet_Box);
    DDX_Control(pDX, IDC_EDIT_P1_IN, m_P1_In_Edit);

    // Strobe values
    DDX_Control(pDX, IDC_STATIC_STROBE_0, m_Strobe0_Text);
    DDX_Control(pDX, IDC_STATIC_STROBE_1, m_Strobe1_Text);
}

BEGIN_MESSAGE_MAP(CPortsDlg, CDialog)
    ON_EN_CHANGE(IDC_EDIT_P0_DIR, OnP0_DirChng)
    ON_EN_CHANGE(IDC_EDIT_P0_OUT, OnP0_OutChng)
    ON_EN_CHANGE(IDC_EDIT_P1_OUT, OnP1_OutChng)
    ON_BN_CLICKED(IDC_BUTTON_P0_DIRSET, OnP0_DirSet)
    ON_BN_CLICKED(IDC_BUTTON_P0_OUTSET, OnP0_OutSet)
    ON_BN_CLICKED(IDC_BUTTON_P1_OUTSET, OnP1_OutSet)
    ON_BN_CLICKED(IDC_BUTTON_P1_DIR, OnP1_DirTog)
    ON_BN_CLICKED(IDCLOSE, OnClose)
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()


// CPortsDlg message handlers

BOOL CPortsDlg::OnInitDialog()
{
    CaDIO_Select adio_select;
    aDIO_Error result;
    char* err_msg = NULL;
    char buff[10];

    CDialog::OnInitDialog();

    // Set the icon for this dialog.  The framework does this automatically
    //  when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);         // Set big icon
    SetIcon(m_hIcon, FALSE);        // Set small icon

    // Set up the user interface
    m_P0_Dir_Edit.SetLimitText(2);
    m_P0_Out_Edit.SetLimitText(2);
    m_P1_Out_Edit.SetLimitText(2);

    sprintf_s(buff, 10, "%02X", m_P0_Direction);
    m_P0_Dir_Edit.SetWindowText(buff);

    sprintf_s(buff, 10, "%02X", m_P0_Output);
    m_P0_Out_Edit.SetWindowText(buff);

    sprintf_s(buff, 10, "%02X", m_P1_Output);
    m_P1_Out_Edit.SetWindowText(buff);

    m_P0_DirSet_Box.EnableWindow(FALSE);
    m_P0_OutSet_Box.EnableWindow(FALSE);
    m_P1_OutSet_Box.EnableWindow(FALSE);

    if(m_P0_Direction == 0x00)
        m_P0_Out_Edit.EnableWindow(FALSE);

    if(m_P1_Direction == ADIO_PORT_DIR_P1_INPUT)
        m_P1_Out_Edit.EnableWindow(FALSE);


    // Let user select the aDIO to use
    do
    {
        adio_select.DoModal();

        if(!adio_select.m_ValidInfo)
        {
            EndDialog(IDCANCEL);
            return FALSE;
        }

        if(adio_select.m_DeviceInfo.availability != ADIO_INTRFC_AVAIL_READY)
        {
            MessageBox( "The aDIO device selected cannot be used.\nTry again.",
                        "Device Unavailable",
                        MB_ICONERROR | MB_OK);
        }
        else
        {
            break;
        }

    } while(TRUE);

    // Open and setup the aDIO
    do
    {
        // Open the aDIO
        result = aDIO_General_Open( adio_select.m_DeviceInfo,
                                    &m_Handle);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Opening the aDIO failed with the following message:\n";
            break;
        }

        // Reset the aDIO
        result = aDIO_General_Reset(m_Handle);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Resetting the aDIO failed with the following message:\n";
            break;
        }

        // Set the directions of both ports
        if(!UpdateDir(false))
            break;

        // Set the output for Port 0
        if(!UpdateOutput(ADIO_PORT_DIO_PORT_0, false))
            break;

        // Set the output for Port 1
        if(!UpdateOutput(ADIO_PORT_DIO_PORT_1, false))
            break;

        // Update the user interface
        if(!RefreshDisplay(false))
            break;

        return TRUE;  // return TRUE unless you set the focus to a control

    } while(0);

    // If we're here there was a failure of some kind
    Cleanup();
    if(err_msg != NULL)
        Fail(result, err_msg, false);
    EndDialog(IDCANCEL);

    return FALSE;
}

bool CPortsDlg::UpdateOutput(aDIO_Port_DIO_Port port, bool quit)
{
    aDIO_Error result;
    char buff[100];

    if(port == ADIO_PORT_DIO_PORT_0)
        result = aDIO_Port_WritePort(m_Handle, port, m_P0_Output);
    else
        result = aDIO_Port_WritePort(m_Handle, port, m_P1_Output);

    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        sprintf_s(
            buff,
            100,
            "Writing the Port %d output failed with the following message:\n",
            port);
        Fail(result, buff, quit);
        return false;
    }

    return true;
}

bool CPortsDlg::UpdateDir(bool quit)
{
    aDIO_Error result;
    uint8 port0;
    aDIO_Port_Dir_P1 port1;
    char buff[512];

    // Set the directions
    result = aDIO_Port_SetDirection(m_Handle, m_P0_Direction, m_P1_Direction);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Setting the Port directions failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    // Read them back and make sure they were set correctly
    result = aDIO_Port_GetDirection(m_Handle, &port0, &port1);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Getting the Port directions failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    if( (port0 != m_P0_Direction) ||
        (port1 != m_P1_Direction))
    {
        sprintf_s(  buff,
                    512,
                    "The direction values read didn't match those written:\n"
                        "Port 0 written: 0x%X\n  Port 0 read: 0x%X\n"
                        "Port 1 written: %d\n  Port 1 read: %d\n",
                    m_P0_Direction,
                    port0,
                    m_P1_Direction,
                    port1);
        Fail(ADIO_ERROR_CODE_SUCCESS, buff, quit);
        return false;
    }

    return true;
}

bool CPortsDlg::RefreshDisplay(bool quit)
{
    aDIO_Error result;
    uint8 value;
    char buff[50];
    aDIO_Port_Strobe_Lev level;

    // Update Port 0 read value

    // 1ms delay needed for back-to-back read/write operations on new cpuModules.
    // Consult the aDIO section of your hardware manual for more information.
    Delay(1);
	
    result = aDIO_Port_ReadPort(m_Handle, ADIO_PORT_DIO_PORT_0, &value);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Getting the value of Port 0 failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    sprintf_s(buff, 50, "%02X", value);
    m_P0_In_Edit.SetWindowText(buff);

    // Update Port 1 read value

    // 1ms delay needed for back-to-back read/write operations on new cpuModules.
    // Consult the aDIO section of your hardware manual for more information.
	Delay(1);

    result = aDIO_Port_ReadPort(m_Handle, ADIO_PORT_DIO_PORT_1, &value);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Getting the value of Port 1 failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    sprintf_s(buff, 50, "%02X", value);
    m_P1_In_Edit.SetWindowText(buff);

    // 1ms delay needed for back-to-back read/write operations on new cpuModules.
    // Consult the aDIO section of your hardware manual for more information.
    Delay(1);
	
    // Update Strobe 0
    result = aDIO_Port_ReadStrobe(m_Handle, ADIO_PORT_STROBE_0, &level);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Getting the Strobe 0 value failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    if(level == ADIO_PORT_STROBE_LEV_LOW)
        m_Strobe0_Text.SetWindowText("Low");
    else
        m_Strobe0_Text.SetWindowText("High");

    // 1ms delay needed for back-to-back read/write operations on new cpuModules.
    // Consult the aDIO section of your hardware manual for more information.
    Delay(1);
	
    // Update Strobe 1
    result = aDIO_Port_ReadStrobe(m_Handle, ADIO_PORT_STROBE_1, &level);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Getting the Strobe 1 value failed "
                    "with the following message:\n",
                quit);
        return false;
    }

    if(level == ADIO_PORT_STROBE_LEV_LOW)
        m_Strobe1_Text.SetWindowText("Low");
    else
        m_Strobe1_Text.SetWindowText("High");

    return true;
}

void CPortsDlg::OnP0_DirChng()
{
    m_P0_DirSet_Box.EnableWindow();
}

void CPortsDlg::OnP0_OutChng()
{
    m_P0_OutSet_Box.EnableWindow();
}

void CPortsDlg::OnP1_OutChng()
{
    m_P1_OutSet_Box.EnableWindow();
}


void CPortsDlg::OnP0_DirSet()
{
    uint8 value;
    char buff[5];

    m_P0_DirSet_Box.EnableWindow(FALSE);

    // Make sure the new port 0 direction is valid
    do
    {
        // Make sure the value itself is valid
        if(!GetHexByteFromEdit(&m_P0_Dir_Edit, &value))
        {
            MessageBox(
                    "The Direction value you entered is invalid.\nTry again.",
                    "Invalid Input",
                    MB_ICONERROR | MB_OK);
        }
        // Make sure to not create an output-to-output connection
        else if((value != 0x00) && (m_P1_Direction != 0x00))
        {
            MessageBox(
                    "You cannot have an output connected to another output!",
                    "Invalid Action",
                    MB_ICONERROR | MB_OK);
        }
        else
        {
            // The value is valid
            break;
        }

        // Return the port 0 edit box to port 0's actual value before returning
        sprintf_s(buff, 5, "%02X", m_P0_Direction);
        m_P0_Dir_Edit.SetWindowText(buff);
        m_P0_DirSet_Box.EnableWindow(FALSE);

        return;

    } while(0);

    // Change direction value
    m_P0_Direction = value;

    // Update direction
    UpdateDir();

    // Check for all bits being inputs
    if(value == 0x00)
    {
        m_P0_Out_Edit.EnableWindow(FALSE);
        m_P0_OutSet_Box.EnableWindow(FALSE);
    }
    else
    {
        m_P0_Out_Edit.EnableWindow();
        m_P0_OutSet_Box.EnableWindow();
    }

    // Update display
    RefreshDisplay();
}

void CPortsDlg::OnP0_OutSet()
{
    uint8 value;
    char buff[5];

    m_P0_OutSet_Box.EnableWindow(FALSE);

    // Read the value from the edit box
    if(!GetHexByteFromEdit(&m_P0_Out_Edit, &value))
    {
        MessageBox( "The Output value you entered is invalid.\nTry again.",
                    "Invalid Input",
                    MB_ICONERROR | MB_OK);

        // Return the port 0 output edit box to port 0's actual value
        sprintf_s(buff, 5, "%02X", m_P0_Output);
        m_P0_Out_Edit.SetWindowText(buff);
        m_P0_OutSet_Box.EnableWindow(FALSE);

        return;
    }

    // Change the internal variable
    m_P0_Output = value;

    // Update the output value
    UpdateOutput(ADIO_PORT_DIO_PORT_0);

    // Update display
    RefreshDisplay();
}

void CPortsDlg::OnP1_OutSet()
{
    uint8 value;
    char buff[5];

    m_P1_OutSet_Box.EnableWindow(FALSE);

    // Read the value from the edit box
    if(!GetHexByteFromEdit(&m_P1_Out_Edit, &value))
    {
        MessageBox( "The Output value you entered is invalid.\nTry again.",
                    "Invalid Input",
                    MB_ICONERROR | MB_OK);

        // Return the port 1 output edit box to port 1's actual value
        sprintf_s(buff, 5, "%02X", m_P1_Output);
        m_P1_Out_Edit.SetWindowText(buff);
        m_P1_OutSet_Box.EnableWindow(FALSE);

        return;
    }

    // Change the internal variable
    m_P1_Output = value;

    // Update the output value
    UpdateOutput(ADIO_PORT_DIO_PORT_1);

    // Update display
    RefreshDisplay();
}


void CPortsDlg::OnP1_DirTog()
{
    m_P1_Dir_Box.EnableWindow(FALSE);

    // Make sure to not create an output-to-output connection
    if((m_P0_Direction != 0x00) && (m_P1_Direction == 0x00))
    {
        MessageBox( "You cannot have an output connected to another output!",
                    "Invalid Action",
                    MB_ICONERROR | MB_OK);
    }
    else
    {
        // Flip the direction of Port 1
        if(m_P1_Direction == ADIO_PORT_DIR_P1_INPUT)
            m_P1_Direction = ADIO_PORT_DIR_P1_OUTPUT;
        else
            m_P1_Direction = ADIO_PORT_DIR_P1_INPUT;

        // Set the user interface as needed
        if(m_P1_Direction == ADIO_PORT_DIR_P1_INPUT)
        {
            m_P1_Dir_Box.SetWindowText("Input");
            m_P1_Out_Edit.EnableWindow(FALSE);
            m_P1_OutSet_Box.EnableWindow(FALSE);
        }
        else
        {
            m_P1_Dir_Box.SetWindowText("Output");
            m_P1_Out_Edit.EnableWindow();
            m_P1_OutSet_Box.EnableWindow();
        }

        // Update direction
        UpdateDir();

        // Update display
        RefreshDisplay();
    }

    m_P1_Dir_Box.EnableWindow();
}

// Handle a Close action
void CPortsDlg::OnClose()
{
    Cleanup();
    CDialog::OnCancel();
}

// Use the enter key to set the values in the text fields
void CPortsDlg::OnOK(void){
    CWnd* pwndCtrl = GetFocus();
    int ctrl_id = pwndCtrl->GetDlgCtrlID();
    
    switch(ctrl_id){
        case IDC_EDIT_P0_DIR:
            OnP0_DirSet();
            break;
        case IDC_EDIT_P0_OUT:
            OnP0_OutSet();
            break;
        case IDC_EDIT_P1_OUT:
            OnP1_OutSet();
            break;
        case IDOK:
            CDialog::OnOK();
            break;
        default:
            break;
    }    
    
}

// Do "clean-up" before dialog finishes
void CPortsDlg::Cleanup()
{
    aDIO_Error result;

    // If m_Handle is NULL, the board hasn't been opened or clean up has already
    //  been done
    if(!m_Handle)
        return;

    // Close the aDIO
    result = aDIO_General_Close(m_Handle);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   ADIO_ERROR_CODE_SUCCESS,
                "Closing the aDIO failed. See the Application Log "\
                    "for the associated error message.",
                false);
    }

    // Mark that clean up is complete
    m_Handle = NULL;
}

// Displays an error message in a small window.
//
// error- If this is not ADIO_ERROR_CODE_SUCCESS, the error code's message is
//      appended to msg.
// msg - This is the example's explanation of the error
// quit - If true, clean up is done before displaying the error message and then
//      the program is exited after displaying the error message.
void CPortsDlg::Fail(aDIO_Error error, char* msg, bool quit)
{
    char final_msg[2048];
    char lib_msg[ADIO_ERROR_MAX_MSG_LENGTH];

    // If there was an error code given, get the last error message from the
    //  aDIO library otherwise just use the provided message.
    if(error != ADIO_ERROR_CODE_SUCCESS)
    {
        // Set a default message, in case getting the error message doesn't work
        sprintf_s(  lib_msg,
                    ADIO_ERROR_MAX_MSG_LENGTH,
                    "\nError message could not be found for error code 0x%X. "
                        "See the Application Log for details.\n",
                    error);

        if(m_Handle)
        {
            // Get the error message
            aDIO_General_GetLastErrorMsg(   m_Handle,
                                            lib_msg,
                                            ADIO_ERROR_MAX_MSG_LENGTH);
        }

        // Combine the retrieved message and the application's error message
        sprintf_s(final_msg, 2048, "%s\n%s", msg, lib_msg);
    }
    else
    {
        // Copy the general error message to the final buffer
        sprintf_s(final_msg, 2048, "%s", msg);
    }

    if(quit)
        Cleanup();

    MessageBox(final_msg, "Error - Aborting", MB_ICONERROR | MB_OK);

    if(quit)
        CDialog::OnCancel();
}


bool CPortsDlg::GetHexByteFromEdit(CEdit* edit, uint8* value)
{
    int numchars;
    char buff[4];
    char temp;

    numchars = edit->LineLength();

    if(numchars < 1 || numchars > 2)
        return false;

    if(!edit->GetWindowText(buff, 3))
        return false;

    *value = 0;

    for(int i = 0; i < numchars; ++i)
    {
        // Shift to the left
        *value *= 16;

        // Add on the next character
        temp = buff[i];

        if(temp >= 'a' && temp <= 'f')
        {
            *value += (temp - 'a') + 10;
        }
        else if(temp >= 'A' && temp <= 'F')
        {
            *value += (temp - 'A') + 10;
        }
        else if(temp >= '0' && temp <= '9')
        {
            *value += temp - '0';
        }
        else
            return false;
    }

    return true;
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CPortsDlg::OnPaint()
{
    if (IsIconic())
    {
        CPaintDC dc(this); // device context for painting

        SendMessage(WM_ICONERASEBKGND,
                    reinterpret_cast<WPARAM>(dc.GetSafeHdc()),
                    0);

        // Center icon in client rectangle
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;

        // Draw the icon
        dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
        CDialog::OnPaint();
    }
}

// The system calls this function to obtain the cursor to display while the
//  user drags the minimized window.
HCURSOR CPortsDlg::OnQueryDragIcon()
{
    return static_cast<HCURSOR>(m_hIcon);
}

// Busy-wait routine which attemts to use the Microsoft QueryPerformanceCounter
void CPortsDlg::Delay(uint32 SleepTimeMS){
    
    LARGE_INTEGER start_count;
    LARGE_INTEGER counts_per_sec;
    LARGE_INTEGER current_count;
    double ticks_per_ms;
    double ticks_passed;
    
    if(QueryPerformanceFrequency(&counts_per_sec) == false){
        // QueryPerformanceFrequency failed. Use Sleep()
        Sleep(1000);
        return;
    }
    ticks_per_ms = static_cast<double>(counts_per_sec.QuadPart) / 1000.0;
    
    QueryPerformanceCounter(&start_count);
    
    while(1){
        // query the counter until at least SleepTimeMS has passed 
        QueryPerformanceCounter(&current_count);
        ticks_passed = static_cast<double>(current_count.QuadPart - start_count.QuadPart);
        if(ticks_passed >= (ticks_per_ms * SleepTimeMS)){
            break;
        }
    }
    
    
}