/*
    Wake-on-aDIO Example

    Please see ReadMe.txt for a description of this example, including
    instructions on setup and use.
*/

#include "stdafx.h"
#include "Wakeup.h"
#include "WakeupDlg.h"
#include "aDIO_Select.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CWakeupDlg dialog



CWakeupDlg::CWakeupDlg(CWnd* pParent /*=NULL*/)
    : CDialog(CWakeupDlg::IDD, pParent)
{
    m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON_RTD);

    m_Handle = NULL;
}

void CWakeupDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);

    DDX_Control(pDX, IDC_BUTTON_STANDBY, m_Standby_Box);
}

BEGIN_MESSAGE_MAP(CWakeupDlg, CDialog)
    ON_BN_CLICKED(IDC_BUTTON_STANDBY, OnStandby)
    ON_BN_CLICKED(IDCLOSE, OnClose)
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()


// CWakeupDlg message handlers

BOOL CWakeupDlg::OnInitDialog()
{
    CaDIO_Select adio_select;
    aDIO_Error result;
    char* err_msg = NULL;

    CDialog::OnInitDialog();

    // Set the icon for this dialog.  The framework does this automatically
    //  when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);         // Set big icon
    SetIcon(m_hIcon, FALSE);        // Set small icon


    // Let user select the aDIO to use
    do
    {
        adio_select.DoModal();

        if(!adio_select.m_ValidInfo)
        {
            EndDialog(IDCANCEL);
            return FALSE;
        }

        // Make sure the aDIO is available, has an IRQ and supports wake-on-aDIO
        if(adio_select.m_DeviceInfo.availability != ADIO_INTRFC_AVAIL_READY)
        {
            MessageBox( "The aDIO device selected cannot be used.\nTry again.",
                        "Device Unavailable",
                        MB_ICONERROR | MB_OK);
        }
        else if(adio_select.m_DeviceInfo.irq == 0)
        {
            MessageBox(
                "The aDIO device selected does not have an IRQ.\nTry again.",
                "Incorrect Device",
                MB_ICONERROR | MB_OK);
        }
        else if(!adio_select.m_DeviceInfo.has_wake)
        {
            MessageBox( "The aDIO device selected doesn't use "\
                            "Wake-on-aDIO.\nTry again.",
                        "Incorrect Device",
                        MB_ICONERROR | MB_OK);
        }
        else
        {
            break;
        }

    } while(TRUE);

    // Open and setup the aDIO
    do
    {
        // Open the aDIO
        result = aDIO_General_Open( adio_select.m_DeviceInfo, &m_Handle);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Opening the aDIO failed with the following message:\n";
            break;
        }

        // Reset the aDIO
        result = aDIO_General_Reset(m_Handle);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Resetting the aDIO failed with the following message:\n";
            break;
        }

        // Set both ports to output
        result = aDIO_Port_SetDirection(m_Handle,
                                        ADIO_PORT_BITS_ALL,
                                        ADIO_PORT_DIR_P1_OUTPUT);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Setting Port directions failed "
                        "with the following message:\n";
            break;
        }

        // Set Port 0 to output 0xFF
        result = aDIO_Port_WritePort(m_Handle, ADIO_PORT_DIO_PORT_0, 0xFF);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Setting Port 0 output failed "
                        "with the following message:\n";
            break;
        }

        // Set Port 1 to output 0xFF
        result = aDIO_Port_WritePort(m_Handle, ADIO_PORT_DIO_PORT_1, 0xFF);
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            err_msg = "Setting Port 1 output failed "
                        "with the following message:\n";
            break;
        }

        // Set up privileges so that we can put the system to sleep
        if(!SetPrivileges())
        {
            result = ADIO_ERROR_CODE_SUCCESS;
            err_msg = "Setup of program privileges failed.\n";
            break;
        }

        return TRUE;  // return TRUE  unless you set the focus to a control

    } while(0);

    // If we're here there was a failure of some kind
    Cleanup();
    if(err_msg != NULL)
        Fail(result, err_msg, false);
    EndDialog(IDCANCEL);

    return FALSE;
}

// Modify the process privileges so it can put the system to sleep
BOOL CWakeupDlg::SetPrivileges()
{
    BOOL status = TRUE;
    HANDLE token;
    TOKEN_PRIVILEGES priv;
    LUID luid;

    if(!OpenProcessToken(   GetCurrentProcess(),
                            TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY,
                            &token))
    {
        return FALSE;
    }

    do
    {
        if(!LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &luid))
        {
            status = FALSE;
            break;
        }

        priv.PrivilegeCount = 1;
        priv.Privileges[0].Luid = luid;
        priv.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;

        if(!AdjustTokenPrivileges(  token,
                                    FALSE,
                                    &priv,
                                    sizeof(TOKEN_PRIVILEGES),
                                    (PTOKEN_PRIVILEGES) NULL,
                                    (PDWORD) NULL))
        {
            status = FALSE;
            break;
        }

        if(GetLastError() == ERROR_NOT_ALL_ASSIGNED)
        {
            status = FALSE;
            break;
        }

    } while(0);

    CloseHandle(token);

    return status;
}


// Handle the 'Go to Standby' button
void CWakeupDlg::OnStandby()
{
    aDIO_Error result;
    aDIO_Interrupt_Int_Type int_type;
    BOOL int_enable, wake_enable;
    uint32 int_count;
    uint8 compare_val, port0_val, port1_val;
    char msg[400];

    // Disable the Standby button
    m_Standby_Box.EnableWindow(FALSE);

    // Turn on Strobe interrupts and Wake-on-aDIO
    result = aDIO_Interrupt_SetConfig(  m_Handle,
                                        TRUE,
                                        ADIO_INTERRUPT_INT_TYPE_STROBE,
                                        TRUE);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Configuring interrupts failed with the following message:\n");
        return;
    }

    // Make sure interrupts and Wake-on-aDIO are configured correctly
    result = aDIO_Interrupt_GetConfig(  m_Handle,
                                        &int_enable,
                                        &int_type,
                                        &wake_enable);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Checking interrupt config failed "
                    "with the following message:\n");
        return;
    }

    if( !int_enable ||
        (int_type != ADIO_INTERRUPT_INT_TYPE_STROBE) ||
        !wake_enable)
    {
        Fail(   ADIO_ERROR_CODE_SUCCESS,
                "The interrupt and Wake-on-aDIO configuration "
                    "was not set correctly.");
        return;
    }

    // Put the system into standby (the function doesn't return until the
    // system wakes back up)
    if(!SetSystemPowerState(TRUE, FALSE))
    {
        Fail(   ADIO_ERROR_CODE_SUCCESS,
                "The system could not be put into standby!");
        return;
    }

    // Wait so the system can wake up and the Wake-on-aDIO event can register
    Sleep(2000);

    // See if the aDIO woke the system
    result = aDIO_Interrupt_Poll(   m_Handle,
                                    ADIO_INTERRUPT_INT_TYPE_STROBE,
                                    &int_count,
                                    &compare_val,
                                    &port0_val,
                                    &port1_val);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Polling the interrupt failed with the following message:\n");
        return;
    }

    // Display an appropriate message box
    if(int_count > 0)
    {
        sprintf_s(  msg,
                    400,
                    "The aDIO awoke the system.\n\n"
                        "Values at the time of the last interrupt were:\n"
                        "  Compare Reg: 0x%02X\n"
                        "  Port 0: 0x%02X\n"
                        "  Port 1: 0x%02X",
                    compare_val,
                    port0_val,
                    port1_val);
    }
    else
    {
        sprintf_s(  msg,
                    400,
                    "The system was awakened, but not by the aDIO!\n\n"
                        "If you tried to wake the system, but weren't able\n"
                        "to, make sure the aDIO's allowed to wake the system.\n"
                        "See the Wakeup example's section in ReadMe.txt.");
    }

    MessageBox(msg, "System Awakened" , MB_OK);

    // Enable the Standby button
    m_Standby_Box.EnableWindow();
}

// Handle a Close action
void CWakeupDlg::OnClose()
{
    Cleanup();
    CDialog::OnCancel();
}

// Do "clean-up" before dialog finishes
void CWakeupDlg::Cleanup()
{
    aDIO_Error result;

    // If m_Handle is NULL, the board hasn't been opened or clean up has already
    //  been done
    if(!m_Handle)
        return;

    // Disable the interrupt and Wake-on-aDIO
    result = aDIO_Interrupt_SetConfig(  m_Handle,
                                        FALSE,
                                        (aDIO_Interrupt_Int_Type)0,
                                        FALSE);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   result,
                "Turning off interrupts failed with the following message:",
                false);
    }

    // Close the aDIO
    result = aDIO_General_Close(m_Handle);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        Fail(   ADIO_ERROR_CODE_SUCCESS,
                "Closing the aDIO failed. See the Application Log "\
                    "for the associated error message.",
                false);
    }

    // Mark that clean up is complete
    m_Handle = NULL;
}

// Displays an error message in a small window.
//
// error- If this is not ADIO_ERROR_CODE_SUCCESS, the error code's message is
//      appended to msg.
// msg - This is the example's explanation of the error
// quit - If true, clean up is done before displaying the error message and then
//      the program is exited after displaying the error message.
void CWakeupDlg::Fail(aDIO_Error error, char* msg, bool quit)
{
    char final_msg[2048];
    char lib_msg[ADIO_ERROR_MAX_MSG_LENGTH];

    // If there was an error code given, get the last error message from the
    //  aDIO library otherwise just use the provided message.
    if(error != ADIO_ERROR_CODE_SUCCESS)
    {
        // Set a default message, in case getting the error message doesn't work
        sprintf_s(  lib_msg,
                    ADIO_ERROR_MAX_MSG_LENGTH,
                    "\nError message could not be found for error code 0x%X. "
                        "See the Application Log for details.\n",
                    error);

        if(m_Handle)
        {
            // Get the error message
            aDIO_General_GetLastErrorMsg(   m_Handle,
                                            lib_msg,
                                            ADIO_ERROR_MAX_MSG_LENGTH);
        }

        // Combine the retrieved message and the application's error message
        sprintf_s(final_msg, 2048, "%s\n%s", msg, lib_msg);
    }
    else
    {
        // Copy the general error message to the final buffer
        sprintf_s(final_msg, 2048, "%s", msg);
    }

    if(quit)
        Cleanup();

    MessageBox(final_msg, "Error - Aborting", MB_ICONERROR | MB_OK);

    if(quit)
        CDialog::OnCancel();
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CWakeupDlg::OnPaint()
{
    if (IsIconic())
    {
        CPaintDC dc(this); // device context for painting

        SendMessage(WM_ICONERASEBKGND,
                    reinterpret_cast<WPARAM>(dc.GetSafeHdc()),
                    0);

        // Center icon in client rectangle
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;

        // Draw the icon
        dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
        CDialog::OnPaint();
    }
}

// The system calls this function to obtain the cursor to display while the
//  user drags the minimized window.
HCURSOR CWakeupDlg::OnQueryDragIcon()
{
    return static_cast<HCURSOR>(m_hIcon);
}
