/**
    @file

    @brief
        This file defines the register map for the aDIO device.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_RegMap_Internal.h 55027 2011-08-15 20:24:09Z jstrouse $
*/

#ifndef __ADIOLIB_REGMAP_INTERNAL_H__
#define __ADIOLIB_REGMAP_INTERNAL_H__

#ifdef __cplusplus
extern "C" {
#endif

//============================================================================
//! @defgroup aDIO_Internal aDIOLib Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_RegMap_Internal aDIOLib Register Map Internal Module
//! @{
//============================================================================

/* 'Port 0 Data' register section */
/**
    Defines the register block where the 'Port 0 Data' register is located.
*/
#define ADIO_REGMAP_INTERNAL_PORT0_BLK      0

/**
    Defines the size of the 'Port 0 Data' register.
*/
#define ADIO_REGMAP_INTERNAL_PORT0_SIZE     ADIO_INTRFC_REG_SIZE_8

/**
    Defines the offset, inside the register block, of the 'Port 0 Data'
    register.
*/
#define ADIO_REGMAP_INTERNAL_PORT0_OFFSET   0x00

/* 'Port 1 Data' register section */
/**
    Defines the register block where the 'Port 1 Data' register is located.
*/
#define ADIO_REGMAP_INTERNAL_PORT1_BLK      0

/**
    Defines the size of the 'Port 1 Data' register.
*/
#define ADIO_REGMAP_INTERNAL_PORT1_SIZE     ADIO_INTRFC_REG_SIZE_8

/**
    Defines the offset, inside the register block, of the 'Port 1 Data'
    register.
*/
#define ADIO_REGMAP_INTERNAL_PORT1_OFFSET   0x01

/* 'Multi-Function' register section */
/**
    Defines the register block where the 'Multi-Function' register is located.
*/
#define ADIO_REGMAP_INTERNAL_MULTIFUNC_BLK      0

/**
    Defines the size of the 'Multi-Function' register.
*/
#define ADIO_REGMAP_INTERNAL_MULTIFUNC_SIZE     ADIO_INTRFC_REG_SIZE_8

/**
    Defines the offset, inside the register block, of the 'Multi-Function'
    register.
*/
#define ADIO_REGMAP_INTERNAL_MULTIFUNC_OFFSET   0x02

/* 'DIO-Control' register section */
/**
    Defines the register block where the 'DIO-Control' register is located.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_BLK                 0

/**
    Defines the size of the 'DIO-Control' register.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_SIZE            ADIO_INTRFC_REG_SIZE_8

/**
    Defines the offset, inside the register block, of the 'DIO-Control'
    register.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_OFFSET              0x03

/**
    Defines a mask for the 'Multi-Function Register Select' bitfield of the
    'DIO-Control' register.

    @note
        This is a guarded bitfield and it must only be changed via the
        IOCTL_ADIO_GUARD_BIT_CONTROL IOCTL.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_MFSELECT_MASK       0x03

/**
    Defines a value for the 'Multi-Function Register Select' bitfield that
    represents the 'Multi-Function' register being placed into 'Clear' mode.

    @note
        This is a guarded bitfield value that is only to be used inside the
        driver.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_MFSELECT_CLEAR      0x00

/**
    Defines a value for the 'Multi-Function Register Select' bitfield that
    represents the 'Multi-Function' register being placed into 'Port 0
    Direction' mode.

    @note
        This is a guarded bitfield value that can be used with the
        IOCTL_ADIO_GUARD_BIT_CONTROL IOCTL.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_MFSELECT_P0DIR      0x01

/**
    Defines a value for the 'Multi-Function Register Select' bitfield that
    represents the 'Multi-Function' register being placed into 'Mask Register'
    mode.

    @note
        This is a guarded bitfield value that can be used with the
        IOCTL_ADIO_GUARD_BIT_CONTROL IOCTL.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_MFSELECT_MASKREG    0x02

/**
    Defines a value for the 'Multi-Function Register Select' bitfield that
    represents the 'Multi-Function' register being placed into 'Compare
    Register' mode.

    @note
        This is a guarded bitfield value that can be used with the
        IOCTL_ADIO_GUARD_BIT_CONTROL IOCTL.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_MFSELECT_COMPARE    0x03

/**
    Defines a mask for the 'Port 1 Direction' bitfield of the 'DIO-Control'
    register.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_P1DIR_MASK          0x04

/**
    Defines a value for the 'Port 1 Direction' bitfield that represents Port 1
    being an input.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_P1DIR_INPUT         0x00

/**
    Defines a value for the 'Port 1 Direction' bitfield that represents Port 1
    being an output.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_P1DIR_OUTPUT        0x04

/**
    Defines a mask for the 'Strobe 1 Status' bitfield of the 'DIO-Control'
    register.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_STROBE1_MASK        0x20

/**
    Defines a mask for the 'Strobe 0 Status' bitfield of the 'DIO-Control'
    register.
*/
#define ADIO_REGMAP_INTERNAL_DIOCONTROL_STROBE0_MASK        0x80

/* 'Wake Control' register section

    This register controls Wake-on-aDIO functionality and it is completely
    controlled by the driver. Use the IOCTL_ADIO_WAKE_CONTROL IOCTL to control
    Wake-on-aDIO.
*/

//============================================================================
//! @} End of Register Map Internal Module
//============================================================================

//============================================================================
//! @} End of Internal Module
//============================================================================


#ifdef __cplusplus
}
#endif

#endif // __ADIOLIB_REGMAP_INTERNAL_H__
